@@ -1,37 +1,19 @@
 #!/usr/bin/env python
 #
-# Copyright (C) 2012, 2013 GRNET S.A. All rights reserved.
+# Copyright (C) 2010-2014 GRNET S.A.
 #
-# Redistribution and use in source and binary forms, with or
-# without modification, are permitted provided that the following
-# conditions are met:
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
 #
-#   1. Redistributions of source code must retain the above
-#      copyright notice, this list of conditions and the following
-#      disclaimer.
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
 #
-#   2. Redistributions in binary form must reproduce the above
-#      copyright notice, this list of conditions and the following
-#      disclaimer in the documentation and/or other materials
-#      provided with the distribution.
-#
-# THIS SOFTWARE IS PROVIDED BY GRNET S.A. ``AS IS'' AND ANY EXPRESS
-# OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
-# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
-# PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL GRNET S.A OR
-# CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
-# USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
-# AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
-# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
-# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
-# POSSIBILITY OF SUCH DAMAGE.
-#
-# The views and conclusions contained in the software and
-# documentation are those of the authors and should not be
-# interpreted as representing official policies, either expressed
-# or implied, of GRNET S.A.
+# You should have received a copy of the GNU General Public License
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 """Unit Tests for the astakos-client module
 
@@ -42,7 +24,11 @@ the astakos client library
 
 import re
 import sys
-import simplejson
+
+try:
+    import simplejson as json
+except ImportError:
+    import json
 
 import astakosclient
 from astakosclient import AstakosClient
@@ -289,10 +275,10 @@ def _req_tokens(conn, method, url, **kwargs):
 
     if 'body' in kwargs:
         # Return endpoints with authenticate info
-        return ("", simplejson.dumps(endpoints_with_info), 200)
+        return ("", json.dumps(endpoints_with_info), 200)
     else:
         # Return endpoints without authenticate info
-        return ("", simplejson.dumps(endpoints), 200)
+        return ("", json.dumps(endpoints), 200)
 
 
 def _req_catalogs(conn, method, url, **kwargs):
@@ -309,7 +295,7 @@ def _req_catalogs(conn, method, url, **kwargs):
         return _request_status_401(conn, method, url, **kwargs)
 
     # Return
-    body = simplejson.loads(kwargs['body'])
+    body = json.loads(kwargs['body'])
     if 'uuids' in body:
         # Return uuid_catalog
         uuids = body['uuids']
@@ -326,7 +312,7 @@ def _req_catalogs(conn, method, url, **kwargs):
         return_catalog = {"displayname_catalog": catalogs, "uuid_catalog": {}}
     else:
         return_catalog = {"displayname_catalog": {}, "uuid_catalog": {}}
-    return ("", simplejson.dumps(return_catalog), 200)
+    return ("", json.dumps(return_catalog), 200)
 
 
 def _req_resources(conn, method, url, **kwargs):
@@ -340,7 +326,7 @@ def _req_resources(conn, method, url, **kwargs):
         return _request_status_400(conn, method, url, **kwargs)
 
     # Return
-    return ("", simplejson.dumps(resources), 200)
+    return ("", json.dumps(resources), 200)
 
 
 def _req_quotas(conn, method, url, **kwargs):
@@ -357,7 +343,7 @@ def _req_quotas(conn, method, url, **kwargs):
         return _request_status_401(conn, method, url, **kwargs)
 
     # Return
-    return ("", simplejson.dumps(quotas), 200)
+    return ("", json.dumps(quotas), 200)
 
 
 def _req_commission(conn, method, url, **kwargs):
@@ -375,22 +361,22 @@ def _req_commission(conn, method, url, **kwargs):
     if method == "POST":
         if 'body' not in kwargs:
             return _request_status_400(conn, method, url, **kwargs)
-        body = simplejson.loads(unicode(kwargs['body']))
+        body = json.loads(unicode(kwargs['body']))
         if re.match('/?'+api_commissions+'$', url) is not None:
             # Issue Commission
             # Check if we have enough resources to give
             if body['provisions'][1]['quantity'] > 420000000:
-                return ("", simplejson.dumps(commission_failure_response), 413)
+                return ("", json.dumps(commission_failure_response), 413)
             else:
                 return \
-                    ("", simplejson.dumps(commission_successful_response), 200)
+                    ("", json.dumps(commission_successful_response), 200)
         else:
             # Issue commission action
             serial = url.split('/')[3]
             if serial == "action":
                 # Resolve multiple actions
                 if body == resolve_commissions_req:
-                    return ("", simplejson.dumps(resolve_commissions_rep), 200)
+                    return ("", json.dumps(resolve_commissions_rep), 200)
                 else:
                     return _request_status_400(conn, method, url, **kwargs)
             else:
@@ -406,12 +392,12 @@ def _req_commission(conn, method, url, **kwargs):
     elif method == "GET":
         if re.match('/?'+api_commissions+'$', url) is not None:
             # Return pending commission
-            return ("", simplejson.dumps(pending_commissions), 200)
+            return ("", json.dumps(pending_commissions), 200)
         else:
             # Return commissions's description
             serial = re.sub('/?' + api_commissions, '', url)[1:]
             if serial == str(57):
-                return ("", simplejson.dumps(commission_description), 200)
+                return ("", json.dumps(commission_description), 200)
             else:
                 return _request_status_404(conn, method, url, **kwargs)
     else:
@@ -781,7 +767,7 @@ class TestCommissions(unittest.TestCase):
         global auth_url
         try:
             client = AstakosClient(token['id'], auth_url)
-            response = client.issue_commission(commission_request)
+            response = client._issue_commission(commission_request)
         except Exception as err:
             self.fail("Shouldn't raise Exception %s" % err)
         self.assertEqual(response, commission_successful_response['serial'])
@@ -795,7 +781,7 @@ class TestCommissions(unittest.TestCase):
         new_request['provisions'][1]['quantity'] = 520000000
         try:
             client = AstakosClient(token['id'], auth_url)
-            client.issue_commission(new_request)
+            client._issue_commission(new_request)
         except QuotaLimit:
             pass
         except Exception as err:
@@ -811,7 +797,8 @@ class TestCommissions(unittest.TestCase):
             client = AstakosClient(token['id'], auth_url)
             response = client.issue_one_commission(
                 "c02f315b-7d84-45bc-a383-552a3f97d2ad",
-                "system", {"cyclades.vm": 1, "cyclades.ram": 30000})
+                {("system", "cyclades.vm"): 1,
+                 ("system", "cyclades.ram"): 30000})
         except Exception as err:
             self.fail("Shouldn't have raised Exception %s" % err)
         self.assertEqual(response, commission_successful_response['serial'])
